@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 
+"""This module contains the logic of resource management"""
+
 import inspect
-import json
-from copy import copy
 from six import with_metaclass
 
 from werkzeug.wrappers import Response
-from flask import request, url_for, make_response, current_app
+from flask import request, url_for, make_response, current_app, jsonify
 from flask.views import MethodView, MethodViewType
 from marshmallow_jsonapi.exceptions import IncorrectTypeError
 from marshmallow import ValidationError
@@ -22,8 +22,10 @@ from flask_rest_jsonapi.data_layers.alchemy import SqlalchemyDataLayer
 
 
 class ResourceMeta(MethodViewType):
+    """Meta class to initilize the data layer and decorators of a resource"""
 
     def __new__(cls, name, bases, d):
+        """Constructor of a resource class"""
         rv = super(ResourceMeta, cls).__new__(cls, name, bases, d)
         if 'data_layer' in d:
             if not isinstance(d['data_layer'], dict):
@@ -46,14 +48,17 @@ class ResourceMeta(MethodViewType):
 
 
 class Resource(MethodView):
+    """Base resource class"""
 
     def __new__(cls):
+        """Constructor of a resource instance"""
         if hasattr(cls, '_data_layer'):
             cls._data_layer.resource = cls
 
         return super(Resource, cls).__new__(cls)
 
     def dispatch_request(self, *args, **kwargs):
+        """Logic of how to handle a request"""
         method = getattr(self, request.method.lower(), None)
         if method is None and request.method == 'HEAD':
             method = getattr(self, 'get', None)
@@ -64,14 +69,21 @@ class Resource(MethodView):
         try:
             response = method(*args, **kwargs)
         except JsonApiException as e:
-            return make_response(json.dumps(jsonapi_errors([e.to_dict()])),
+            return make_response(jsonify(jsonapi_errors([e.to_dict()])),
                                  e.status,
                                  headers)
         except Exception as e:
             if current_app.config['DEBUG'] is True:
                 raise e
-            exc = JsonApiException('', str(e))
-            return make_response(json.dumps(jsonapi_errors([exc.to_dict()])),
+            exc = JsonApiException(getattr(e, 'detail', str(e)),
+                                   source=getattr(e, 'source', ''),
+                                   title=getattr(e, 'title', None),
+                                   status=getattr(e, 'status', None),
+                                   code=getattr(e, 'code', None),
+                                   id_=getattr(e, 'id', None),
+                                   links=getattr(e, 'links', None),
+                                   meta=getattr(e, 'meta', None))
+            return make_response(jsonify(jsonapi_errors([exc.to_dict()])),
                                  exc.status,
                                  headers)
 
@@ -82,7 +94,7 @@ class Resource(MethodView):
         if not isinstance(response, tuple):
             if isinstance(response, dict):
                 response.update({'jsonapi': {'version': '1.0'}})
-            return make_response(json.dumps(response), 200, headers)
+            return make_response(jsonify(response), 200, headers)
 
         try:
             data, status_code, headers = response
@@ -98,15 +110,15 @@ class Resource(MethodView):
         if isinstance(data, dict):
             data.update({'jsonapi': {'version': '1.0'}})
 
-        return make_response(json.dumps(data), status_code, headers)
+        return make_response(jsonify(data), status_code, headers)
 
 
 class ResourceList(with_metaclass(ResourceMeta, Resource)):
+    """Base class of a resource list manager"""
 
     @check_method_requirements
     def get(self, *args, **kwargs):
-        """Retrieve a collection of objects
-        """
+        """Retrieve a collection of objects"""
         self.before_get(args, kwargs)
 
         qs = QSManager(request.args, self.schema)
@@ -126,18 +138,18 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
         add_pagination_links(result,
                              objects_count,
                              qs,
-                             url_for(self.view, **view_kwargs))
+                             url_for(self.view, _external=True, **view_kwargs))
 
         result.update({'meta': {'count': objects_count}})
 
         self.after_get(result)
+
         return result
 
     @check_method_requirements
     def post(self, *args, **kwargs):
-        """Create an object
-        """
-        json_data = request.get_json()
+        """Create an object"""
+        json_data = request.get_json() or {}
 
         qs = QSManager(request.args, self.schema)
 
@@ -172,28 +184,34 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
         obj = self._data_layer.create_object(data, kwargs)
 
         result = schema.dump(obj).data
+
         self.after_post(result)
+
         return result, 201, {'Location': result['data']['links']['self']}
 
     def before_get(self, args, kwargs):
+        """Hook to make custom work before get method"""
         pass
 
     def after_get(self, result):
+        """Hook to make custom work after get method"""
         pass
 
     def before_post(self, args, kwargs, data=None):
+        """Hook to make custom work before post method"""
         pass
 
     def after_post(self, result):
+        """Hook to make custom work after post method"""
         pass
 
 
 class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
+    """Base class of a resource detail manager"""
 
     @check_method_requirements
     def get(self, *args, **kwargs):
-        """Get object details
-        """
+        """Get object details"""
         self.before_get(args, kwargs)
 
         obj = self._data_layer.get_object(kwargs)
@@ -208,13 +226,13 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
         result = schema.dump(obj).data
 
         self.after_get(result)
+
         return result
 
     @check_method_requirements
     def patch(self, *args, **kwargs):
-        """Update an object
-        """
-        json_data = request.get_json()
+        """Update an object"""
+        json_data = request.get_json() or {}
 
         qs = QSManager(request.args, self.schema)
         schema_kwargs = getattr(self, 'patch_schema_kwargs', dict())
@@ -247,9 +265,11 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
             return errors, 422
 
         if 'id' not in json_data['data']:
-            raise BadRequest('/data/id', 'Missing id in "data" node')
+            raise BadRequest('Missing id in "data" node',
+                             source={'pointer': '/data/id'})
         if json_data['data']['id'] != str(kwargs[self.data_layer.get('url_field', 'id')]):
-            raise BadRequest('/data/id', 'Value of id does not match the resource identifier in url')
+            raise BadRequest('Value of id does not match the resource identifier in url',
+                             source={'pointer': '/data/id'})
 
         self.before_patch(args, kwargs, data=data)
 
@@ -259,46 +279,54 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
         result = schema.dump(obj).data
 
         self.after_patch(result)
+
         return result
 
     @check_method_requirements
     def delete(self, *args, **kwargs):
-        """Delete an object
-        """
+        """Delete an object"""
         self.before_delete(args, kwargs)
 
         obj = self._data_layer.get_object(kwargs)
         self._data_layer.delete_object(obj, kwargs)
 
         result = {'meta': {'message': 'Object successfully deleted'}}
+
         self.after_delete(result)
+
         return result
 
     def before_get(self, args, kwargs):
+        """Hook to make custom work before get method"""
         pass
 
     def after_get(self, result):
+        """Hook to make custom work after get method"""
         pass
 
     def before_patch(self, args, kwargs, data=None):
+        """Hook to make custom work before patch method"""
         pass
 
     def after_patch(self, result):
+        """Hook to make custom work after patch method"""
         pass
 
     def before_delete(self, args, kwargs):
+        """Hook to make custom work before delete method"""
         pass
 
     def after_delete(self, result):
+        """Hook to make custom work after delete method"""
         pass
 
 
 class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
+    """Base class of a resource relationship manager"""
 
     @check_method_requirements
     def get(self, *args, **kwargs):
-        """Get a relationship details
-        """
+        """Get a relationship details"""
         self.before_get(args, kwargs)
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
@@ -320,33 +348,34 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
             result['included'] = serialized_obj.data.get('included', dict())
 
         self.after_get(result)
+
         return result
 
     @check_method_requirements
     def post(self, *args, **kwargs):
-        """Add / create relationship(s)
-        """
-        json_data = request.get_json()
+        """Add / create relationship(s)"""
+        json_data = request.get_json() or {}
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
 
         if 'data' not in json_data:
-            raise BadRequest('/data', 'You must provide data with a "data" route node')
+            raise BadRequest('You must provide data with a "data" route node', source={'pointer': '/data'})
         if isinstance(json_data['data'], dict):
             if 'type' not in json_data['data']:
-                raise BadRequest('/data/type', 'Missing type in "data" node')
+                raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
             if 'id' not in json_data['data']:
-                raise BadRequest('/data/id', 'Missing id in "data" node')
+                raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
             if json_data['data']['type'] != related_type_:
-                raise InvalidType('/data/type', 'The type field does not match the resource type')
+                raise InvalidType('The type field does not match the resource type', source={'pointer': '/data/type'})
         if isinstance(json_data['data'], list):
             for obj in json_data['data']:
                 if 'type' not in obj:
-                    raise BadRequest('/data/type', 'Missing type in "data" node')
+                    raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
                 if 'id' not in obj:
-                    raise BadRequest('/data/id', 'Missing id in "data" node')
+                    raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
                 if obj['type'] != related_type_:
-                    raise InvalidType('/data/type', 'The type provided does not match the resource type')
+                    raise InvalidType('The type provided does not match the resource type',
+                                      source={'pointer': '/data/type'})
 
         self.before_post(args, kwargs, json_data=json_data)
 
@@ -367,34 +396,36 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
         result = schema.dump(obj_).data
         if result.get('links', {}).get('self') is not None:
             result['links']['self'] = request.path
+
         self.after_post(result)
+
         return result, 200
 
     @check_method_requirements
     def patch(self, *args, **kwargs):
-        """Update a relationship
-        """
-        json_data = request.get_json()
+        """Update a relationship"""
+        json_data = request.get_json() or {}
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
 
         if 'data' not in json_data:
-            raise BadRequest('/data', 'You must provide data with a "data" route node')
+            raise BadRequest('You must provide data with a "data" route node', source={'pointer': '/data'})
         if isinstance(json_data['data'], dict):
             if 'type' not in json_data['data']:
-                raise BadRequest('/data/type', 'Missing type in "data" node')
+                raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
             if 'id' not in json_data['data']:
-                raise BadRequest('/data/id', 'Missing id in "data" node')
+                raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
             if json_data['data']['type'] != related_type_:
-                raise InvalidType('/data/type', 'The type field does not match the resource type')
+                raise InvalidType('The type field does not match the resource type', source={'pointer': '/data/type'})
         if isinstance(json_data['data'], list):
             for obj in json_data['data']:
                 if 'type' not in obj:
-                    raise BadRequest('/data/type', 'Missing type in "data" node')
+                    raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
                 if 'id' not in obj:
-                    raise BadRequest('/data/id', 'Missing id in "data" node')
+                    raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
                 if obj['type'] != related_type_:
-                    raise InvalidType('/data/type', 'The type provided does not match the resource type')
+                    raise InvalidType('The type provided does not match the resource type',
+                                      source={'pointer': '/data/type'})
 
         self.before_patch(args, kwargs, json_data=json_data)
 
@@ -415,34 +446,36 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
         result = schema.dump(obj_).data
         if result.get('links', {}).get('self') is not None:
             result['links']['self'] = request.path
+
         self.after_patch(result)
+
         return result, 200
 
     @check_method_requirements
     def delete(self, *args, **kwargs):
-        """Delete relationship(s)
-        """
-        json_data = request.get_json()
+        """Delete relationship(s)"""
+        json_data = request.get_json() or {}
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
 
         if 'data' not in json_data:
-            raise BadRequest('/data', 'You must provide data with a "data" route node')
+            raise BadRequest('You must provide data with a "data" route node', source={'pointer': '/data'})
         if isinstance(json_data['data'], dict):
             if 'type' not in json_data['data']:
-                raise BadRequest('/data/type', 'Missing type in "data" node')
+                raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
             if 'id' not in json_data['data']:
-                raise BadRequest('/data/id', 'Missing id in "data" node')
+                raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
             if json_data['data']['type'] != related_type_:
-                raise InvalidType('/data/type', 'The type field does not match the resource type')
+                raise InvalidType('The type field does not match the resource type', source={'pointer': '/data/type'})
         if isinstance(json_data['data'], list):
             for obj in json_data['data']:
                 if 'type' not in obj:
-                    raise BadRequest('/data/type', 'Missing type in "data" node')
+                    raise BadRequest('Missing type in "data" node', source={'pointer': '/data/type'})
                 if 'id' not in obj:
-                    raise BadRequest('/data/id', 'Missing id in "data" node')
+                    raise BadRequest('Missing id in "data" node', source={'pointer': '/data/id'})
                 if obj['type'] != related_type_:
-                    raise InvalidType('/data/type', 'The type provided does not match the resource type')
+                    raise InvalidType('The type provided does not match the resource type',
+                                      source={'pointer': '/data/type'})
 
         self.before_delete(args, kwargs, json_data=json_data)
 
@@ -461,16 +494,17 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
         result = schema.dump(obj_).data
         if result.get('links', {}).get('self') is not None:
             result['links']['self'] = request.path
+
         self.after_delete(result)
+
         return result, status_code
 
     def _get_relationship_data(self):
-        """Get useful data for relationship management
-        """
+        """Get useful data for relationship management"""
         relationship_field = request.path.split('/')[-1]
 
-        if relationship_field not in get_relationships(self.schema).values():
-            raise RelationNotFound('', "{} has no attribute {}".format(self.schema.__name__, relationship_field))
+        if relationship_field not in get_relationships(self.schema):
+            raise RelationNotFound("{} has no attribute {}".format(self.schema.__name__, relationship_field))
 
         related_type_ = self.schema._declared_fields[relationship_field].type_
         related_id_field = self.schema._declared_fields[relationship_field].id_field
@@ -479,25 +513,33 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
         return relationship_field, model_relationship_field, related_type_, related_id_field
 
     def before_get(self, args, kwargs):
+        """Hook to make custom work before get method"""
         pass
 
     def after_get(self, result):
+        """Hook to make custom work after get method"""
         pass
 
     def before_post(self, args, kwargs, json_data=None):
+        """Hook to make custom work before post method"""
         pass
 
     def after_post(self, result):
+        """Hook to make custom work after post method"""
         pass
 
     def before_patch(self, args, kwargs, json_data=None):
+        """Hook to make custom work before patch method"""
         pass
 
     def after_patch(self, result):
+        """Hook to make custom work after patch method"""
         pass
 
     def before_delete(self, args, kwargs, json_data=None):
+        """Hook to make custom work before delete method"""
         pass
 
     def after_delete(self, result):
+        """Hook to make custom work after delete method"""
         pass
